program fkeycheck;
{ User-friendly test for function keys, using ncurses and ncursesPlus  }
{$ignore-function-results}  { this avoids need for retcode:= on most ncurses library calls }
{ compile for GPC: gp fkeycheck }
{ W. Kempton, rev 2 December 2005 }

import ncurses;
       ncursesPlus; { imports: keylabelmax, labstrtype, KeyLabels, HasKeySet, TermNamePS }

const
  maxrowhead = 18;
var
  c1, c2: chtype; { chtype is the ncurses "char" type (actually integer) returned by getch etc }
  i, row, col: integer;
  linecount, colcount: cint; { from ncurses.pas  }
  retcode: Curses_int; { ncurses return code (not used consistently in this program) }

function IsFKey(c: chtype): boolean;
{ return true iff c is a function key, }
begin
  IsFKey :=  ( c >=  KEY_MIN ) AND ( c<= KEY_MAX );
end { IsFKey };


procedure AddInt(cc: integer);
{ puts a three-digit integer in the output buffer }
begin
        addch(ord('0') + cc div 100);
        cc :=  cc mod 100;
        addch(ord('0') + (cc div 10) );
        cc := cc mod 10;
        addch(ord('0') + (cc ) );
end { AddInt };

begin { fkeycheck }
{ ncurses initialization }
   initscr;
   clear; { initscr also clears in most, not all, implementations }
   noecho;
   raw;   { no line buffering, no interpreting interrupt, quit, suspend, and flow control  characters }
   keypad(stdscr, true);  { enable ncurses' recognition of F keys }
   getmaxyx(stdscr, linecount, colcount);  { max row and col }  {  not used here }

   move(0,0); addstr('terminfo terminal name is "');
   addstr(TermNamePS); addstr('"; it defines these keys:'); 
   row := 1; col:= 0;
   for c1 := KEY_MIN to KEY_MAX do
      begin
        if c1 in HasKeySet then 
           begin
             retcode := move(row,col);
             if odd(row) 
               then retcode := attrset(A_REVERSE)
                else retcode := attrset(A_UNDERLINE)  ;
             for i:= 1 to length(KeyLabels[c1])  do
                retcode := addch(ord(KeyLabels[c1,i]));
             retcode := attrset(A_NORMAL);
           row := row + 1;
           if row > maxrowhead then 
               begin 
                   row := 1; 
                   col:=col+12; retcode := refresh; 
               end;
          end;
     end;
   retcode := refresh;
  
   row := maxrowhead+1; move(row,0); 
   addstr('Press keys to test; last key pressed shows');
   row := row+1; move(row,0);
   addstr('  (Commands:  "K"/"k" keypad on/off; "q" to quit.)');
   row := row + 1;  { row..row+2;  label starts column 0, value will start column 11 }
   move(row, 0);   addstr('   keypad: T');
   move(row+1,0);  addstr('    F key:');
   move(row+2,0);  addstr('non-F key:');
   refresh;
   repeat
     c1 := getch;
     if IsFKey(c1) then move(row+1,11) else move(row+2,11);
     if IsFKey(c1) 
        then begin 
                attrset(A_REVERSE); addstr(KeyLabels[c1]); attrset(A_NORMAL); addstr(' '); clrtoeol; 
             end
        else begin 
               addstr('"'); addstr(CString2String(unctrl(c1))); addstr('" ');
               addstr('   (value='); AddInt(c1); addstr(') ');
             end;
      if c1 in [ord('k'), ord('K')] then 
            begin  keypad(stdscr, chr(c1)='K');
               move(row,11); if c1=ord('K') then addstr('T') else addstr('F');
            end;
     refresh; 
   until ord('q') = c1;
   { last info should be left on screen }
   endwin; 
end.

