program testkey;
{ test ncurses for f key and character input  }
{ compile for GPC: gp --ignore-function-results testkey
}

uses ncurses;

var
  c1, c2: chtype; { chtype is the ncurses "char" type (actually integer) returned by getch etc }
  i, line, col: integer;
  linecount, colcount: cint; { from ncurses.pas  }

function IsFKey(c: chtype): boolean;
{ return true iff c is a function key, }
begin
  IsFKey :=  ( c >=  KEY_MIN ) AND ( c<= KEY_MAX );
end { IsFKey };

function KeyNameClean(c: chtype): pchar;
  { return a user-friendly key name, e.g. "KEY_F(2)"  => "F2" ;  Meta-A not garbled  }
  { an improved version of this is in module ncursesPlus.pas  }
var sx: string[120]; ix: integer;
begin
  if (c > 127) and (c < 256) 
    then sx := 'M-' + CString2String(unctrl(c mod 128))
    else
      begin
        sx := CString2String(keyname(c));
        ix :=  index(sx,'KEY_');
        if (ix=1) and (length(sx)>4) then sx :=  substr(sx,5);
        ix := index(sx,'(');
        if (ix>1) and (ix+1<length(sx)) then 
          begin  
            sx := sx[1..ix-1] + sx[ix+1..length(sx)];  { trim the "(" }
            if sx[length(sx)] = ')' then sx :=substr(sx,1,length(sx)-1);
          end;
     end;
  KeyNameClean := sx;
end { UserKeyName };

procedure AddInt(cc: integer);
{ puts a three-digit integer in the output buffer }
begin
        addch(ord('0') + cc div 100);
        cc :=  cc mod 100;
        addch(ord('0') + (cc div 10) );
        cc := cc mod 10;
        addch(ord('0') + (cc ) );
end { AddInt };

begin
  writeln('About to set up ncurses ..'); { if setup fails, this stays on screen }
  writeln('pre-initscr terminal name is said to be ', CString2String(termname));
  writeln('type RETURN, and you will enter RAW mode...  exit only as instructed (no ^C)');
  readln;
{ ncurses initialization }
   initscr;
   clear; { initscr also clears in most, not all, implementations }
   noecho;
   { cbreak; }    { No line buffering, pass on as typed including erase, but process ^C and ^Z }
   raw;   { raw additionally does not interpret interrupt, quit, suspend, and flow control  characters }
   keypad(stdscr, true);  { enable ncurses' recognition of F keys }
  move(0,0); addstr('post-initscr, terminal name is said to be ');
   addstr(termname); 
   move(1,0);
   getbegyx(stdscr, linecount, colcount);
   addstr('begin line, begin col = ');AddInt(linecount); addstr(', '); AddInt(colcount); refresh;
   getmaxyx(stdscr, linecount, colcount);
   move(2,0);
   addstr('linecount, colcount = ');  AddInt(linecount); addstr(', '); AddInt(colcount); refresh;
   c2 := getch; { pause for user }
   clear;
   line := -1;  col:= 0; 
   for c1 := KEY_MIN to KEY_MAX do 
     begin
       line := line + 1;
       if line>=linecount then 
            begin refresh; line:=0; col:=col+20;
               if col+10 >= colcount then 
                  begin refresh; c2:=getch; col:=0; clear;  end;
             end;
       move( line, col);   AddInt(c1);
       addstr(': ');  addstr(keyname(c1)); 
       if has_key(c1) then addstr('*');
     end;
   refresh;
   c1 := getch;
   clear;
   move(0,0);
   line := -1;  col:= 0; 
   for c1 := 0 to 127 do
     begin
       line := line + 1;
       if line>=linecount then 
            begin refresh; line:=0; col:=col+20;
               if col+10 >= colcount then 
                  begin refresh; c2:=getch; col:=0; clear;  end;
             end;
       move( line, col);   AddInt(c1);
       addstr(': '+  CString2String(unctrl(c1))); 
       { if has_key(c1) then addstr('*'); }
     end;
   refresh;  c1 := getch;
   clear;
   move(0,0); { line, col, both zero-origin }
   addstr('type characters and F keys; "M" meta on, "m" meta off; "q" to quit: ');
   refresh;
   repeat
     c1 := getch;
     move(1,0); clrtoeol;  addstr('Is F Key=');
     if IsFKey(c1) then addstr('True  ') else addstr('False ');
     addstr('keynameClean="');  
     addstr(KeyNameClean(c1));
     addstr('"   value='); AddInt(c1);
     addstr('   chr="'); addch(c1); addstr('" ');
     if c1 = ord('M') then meta(stdscr,true)
     else if c1 = ord('m') then meta(stdscr,false);
    refresh; 
   until ord('q') = c1;
   { last info should be left on screen }
   endwin; 
end.

